@@ -38,7 +38,9 @@ static FILE *getout()
 
 void close_debug()
 {
-     fclose(debug_output);
+     FILE *f;
+     if ((f = getout()))
+	  fclose(f);
      debug_output = NULL;
 }
 
