@@ -157,7 +157,7 @@ void my_play(char **argv)
   }else{
     state=STATE_PLAY;
     if(!repeats) repeats=igetenv("repeats",1);
-    repeats--;
+    if(repeats != MAXINT) repeats--;
   }
 }
 
