@@ -78,6 +78,9 @@ static void place_window()
 	  XSync(display, FALSE);
      }
 
+     XMapWindow(display, victim);
+     XSync(display, FALSE);
+
      if (flags & H_FILL) {
 	  fill_window(display, victim, w, h);
      } else if (flags & H_MAXASPECT) {
@@ -252,19 +255,19 @@ static int event_swallow()
      {
 	  /* got a structure event */
 	  D("Got Structure Event of type %x\n", evt.type);
-	  if (evt.type == ReparentNotify)
+	  if (evt.type == CreateNotify)
 	  {
 		  
 	       /* looks promising, let's see if it's the one we're looking for */
-	       D("Found ReparentNotify Event %x\n", evt.type);
+	       D("Found CreateNotify Event %x\n", evt.type);
 
-	       if (!XGetClassHint(display, evt.xreparent.window, &windowclass))
+	       if (!XGetClassHint(display, evt.xcreatewindow.window, &windowclass))
 	       {
 		    D("Could Not get Class Hint, trying the next one");
 		    continue;
 	       }
 
-	       if (!XFetchName(display, evt.xreparent.window, &windowname))
+	       if (!XFetchName(display, evt.xcreatewindow.window, &windowname))
 	       {
 		    D("Could Not Get Window Name, trying the next one");
 		    continue;
@@ -279,7 +282,7 @@ static int event_swallow()
 			strncmp(victimname, windowclass.res_class, len) == 0 ||
 			strncmp(victimname, windowclass.res_name, len) == 0)
 		    {
-			 victim = evt.xreparent.window;
+			 victim = evt.xcreatewindow.window;
 			 found = 1;
 			 D("Found it!! Victim=%x\n", victim);
 			 D("Flags %x\n", flags);
