@@ -44,6 +44,7 @@ typedef struct data
      uint16 mode;
      char *mimetype;
      char *href;
+     char *mms;
      char autostart;
      int num_arguments;
      struct argument *args;
@@ -73,8 +74,8 @@ typedef struct handle
 typedef struct
 {
      char *name;
-     int flag;
-} options_t;
+     int value;
+} flag_t;
 
 /*****************************************************************************
  * Global variables
@@ -166,7 +167,7 @@ static void run(NPP instance, const char *file)
 	     THIS->repeats,
 	     (unsigned long int) THIS->windata.window,
 	     (int) THIS->windata.x,
-	      (int) THIS->windata.y,
+	     (int) THIS->windata.y,
 	     (int) THIS->windata.width,
 	     (int) THIS->windata.height);
 
@@ -187,7 +188,7 @@ static void run(NPP instance, const char *file)
      my_putenv(buffer, &offset, "hexwindow", foo);
 
      snprintf(foo, sizeof(foo), "%ld", (long)THIS->repeats);
-     my_putenv(buffer, &offset, "repeat", foo);
+     my_putenv(buffer, &offset, "repeats", foo);
 
      snprintf(foo, sizeof(foo), "%ld", (long)THIS->windata.width);
      my_putenv(buffer, &offset, "width", foo);
@@ -298,17 +299,17 @@ static void filter_previous_handler()
 }
 
 /*****************************************************************************
- * Match a word to an option of the type swallow(name).
+ * Match a word to a flag of the type swallow(name).
  *****************************************************************************/
-static char *gobble_winname(char *x, char *option, command_t *c)
+static char *get_winname(char *x, char *flag, command_t *c)
 {
      char *end;
  
-     x += strlen(option);
+     x += strlen(flag);
      while (isspace(*x)) x++;
      if (*x != '(')
      {
-	  fprintf(stderr,"Expected '(' after '%s'\n", option);
+	  fprintf(stderr,"Expected '(' after '%s'\n", flag);
 	  return NULL;
      }
      x++;
@@ -325,19 +326,20 @@ static char *gobble_winname(char *x, char *option, command_t *c)
 }
 
 /*****************************************************************************
- * Match a word to a single word option.
+ * Match a word to a single-word flag.
  *****************************************************************************/
-static int gobble(char *line, char *kw)
+__inline
+static int match(char *line, char *kw)
 {
      return !strncasecmp(line,kw,strlen(kw)) && !isalnum(line[strlen(kw)]);
 }
 
 /*****************************************************************************
- * Scan a line for all the possible options.
+ * Scan a line for all the possible flags.
  *****************************************************************************/
-static int parse_options(char **x, command_t *commandp)
+static int parse_flags(char **x, command_t *commandp)
 {
-     static options_t options[] = {
+     static flag_t flags[] = {
 	  { "repeat", 		H_REPEATCOUNT 	},
 	  { "loop", 		H_LOOP 		},
 	  { "stream", 		H_STREAM 	},
@@ -351,29 +353,29 @@ static int parse_options(char **x, command_t *commandp)
 	  { "noembed",          H_NOEMBED       },
 	  { NULL, 		0 		}
      };
-     options_t *opt;
+     flag_t *f;
 
-     for (opt = options; opt->name; opt++)
+     for (f = flags; f->name; f++)
      {
-	  if (gobble(*x, opt->name))
+	  if (match(*x, f->name))
 	  {
-	       *x += strlen(opt->name);
-	       commandp->flags |= opt->flag;
+	       *x += strlen(f->name);
+	       commandp->flags |= f->value;
 	       return 1;
 	  }
-	  if (gobble(*x, "swallow"))
+	  if (match(*x, "swallow"))
 	  {
 	       commandp->flags |= H_SWALLOW;
-	       if ((*x = gobble_winname(*x, "swallow", commandp)))
+	       if ((*x = get_winname(*x, "swallow", commandp)))
 		    return 1;
 	  }
-	  if (gobble(*x, "event_swallow"))
+	  if (match(*x, "event_swallow"))
 	  {
 	       commandp->flags |= H_EVENT_SWALLOW;
-	       if ((*x = gobble_winname(*x, "event_swallow", commandp)))
+	       if ((*x = get_winname(*x, "event_swallow", commandp)))
 		    return 1;
 	  }
-	  if (gobble(*x, "controls"))
+	  if (match(*x, "controls"))
 	  {
 	       commandp->flags |= H_CONTROLS | H_SWALLOW | H_FILL;
 	       commandp->winname = strdup("mozplugger-controller");
@@ -443,7 +445,7 @@ static void read_config(FILE *f)
 			 x++;
 			 break;
 		    default:
-			 if (!parse_options(&x, *commandp))
+			 if (!parse_flags(&x, *commandp))
 			 {
 			      D("Unknown directive: %s\n", x);
 			      fprintf(stderr,"Unknown directive: %s\n", x);
@@ -612,15 +614,15 @@ static void do_read_config(void)
 
      if (find("mozplugger-helper")) helper_fname = "mozplugger-helper";
      find_helper_file("mozplugger-helper", find_plugger_helper_cb, 0);
-     
-     if (find("mozplugger-controller")) controller_fname = "mozplugger-controller";
-     find_helper_file("mozplugger-controller", find_plugger_controller_cb, 0);
 
      if (!helper_fname)
      {
 	  fprintf(stderr, "MozPlugger: Unable to find the mozplugger helper.\n");
 	  return;
      }
+     
+     if (find("mozplugger-controller")) controller_fname = "mozplugger-controller";
+     find_helper_file("mozplugger-controller", find_plugger_controller_cb, 0);
 
      D("do_read_config done\n");
 }
@@ -805,7 +807,7 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
  * Convert a string to an integer.
  * The string can be true, false, yes or no.
  *****************************************************************************/
-int my_atoi(char *s, int my_true, int my_false)
+static int my_atoi(char *s, int my_true, int my_false)
 {
      switch (s[0])
      {
@@ -821,7 +823,7 @@ int my_atoi(char *s, int my_true, int my_false)
 }
 
 /*****************************************************************************
- * Initiate another instance of mozplugger, it is important to know
+ * Initialize another instance of mozplugger. It is important to know
  * that there might be several instances going at one time.
  *****************************************************************************/
 NPError NPP_New(NPMIMEType pluginType,
@@ -883,7 +885,7 @@ NPError NPP_New(NPMIMEType pluginType,
 	  {
 	       src_attr = e;
 	  }
-	  /* copy the option to put it into the environement later */
+	  /* copy the flag to put it into the environement later */
 	  D("VAR_%s=%s\n", argn[e], argv[e]);
 	  if (!(THIS->args[e].var = (char *)malloc(strlen(argn[e]) + 5)))
 	       return NPERR_OUT_OF_MEMORY_ERROR;
@@ -892,17 +894,31 @@ NPError NPP_New(NPMIMEType pluginType,
 	  THIS->args[e].value = argv[e] ? strdup(argv[e]) : NULL;
      }
 
-     /* Special case for quicktime. If there's an href attribute, we 
+     /* Special case for quicktime. If there's an href or qtsrc attribute, we 
 	want that instead of src but we HAVE to have a src first. */
      for (e = 0; e < argc; e++)
      {
-          if (!strcasecmp("href", argn[e]) && src_attr > -1)
+          D("arg %d(%s): %s\n",e,argn[e],argv[e]);
+          if ((!strcasecmp("href", argn[e]) ||
+	       !strcasecmp("qtsrc", argn[e])) &&
+	      src_attr > -1 &&
+	      !THIS->href)
           {
 	       if (!(THIS->href = strdup(argv[e])))
 		    return NPERR_OUT_OF_MEMORY_ERROR;
           }
      }
 
+     if (src_attr > -1)
+     {
+	  char *url = argv[src_attr];
+	  if (!strncmp(url, "mms://", 6) || !strncmp(url, "mmst://", 7))
+	  {
+	       D("Detected MMS\n");
+	       THIS->mms = strdup(url);
+	  }
+     }
+
      D("New finished\n");
 
      return NPERR_NO_ERROR;
@@ -931,10 +947,12 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
 	       free((char *)THIS->args[e].var);
 	       free((char *)THIS->args[e].value);
 	  }
+
 	  NPN_MemFree((char *)THIS->args);
 
 	  free(THIS->mimetype);
 	  free(THIS->href);
+	  free(THIS->mms);
 
 	  NPN_MemFree(THIS);
 	  THIS = NULL;
@@ -1028,15 +1046,11 @@ int32 NPP_WriteReady(NPP instance, NPStream *stream)
 }
 
 /*****************************************************************************
- * Called after NPP_NewStream if *stype = NP_ASFILEONLY.
+ * Check that no child is already running before forking one.
  *****************************************************************************/
-void NPP_StreamAsFile(NPP instance,
-		      NPStream *stream,
-		      const char* fname)
+static void new_child(NPP instance, const char* fname)
 {
-     D("StreamAsFile\n");
-
-     if (!instance || !tempfile || !fname)
+     if (!instance || !fname)
 	  return;
 
      if (THIS->pid != -1)
@@ -1056,48 +1070,53 @@ void NPP_StreamAsFile(NPP instance,
      }
 }
 
+/*****************************************************************************
+ * Called after NPP_NewStream if *stype = NP_ASFILEONLY.
+ *****************************************************************************/
+void NPP_StreamAsFile(NPP instance,
+		      NPStream *stream,
+		      const char* fname)
+{
+     D("StreamAsFile\n");
+     new_child(instance, fname);
+}
+
 /*****************************************************************************
  * Find and resize the embedded window.
  *****************************************************************************/
-int resize_window(int flags, Display *display, Window *window, int w, int h)
+void resize_window(int flags, Display *display, Window *window, int w, int h)
 {
      Window *useme = NULL;
      Window root, parent;
      Window *children = NULL;
      unsigned int num_children;
 
-     if (((flags & H_SWALLOW) || (flags & H_EVENT_SWALLOW)) &&
-	 ((flags & H_FILL) || (flags & H_MAXASPECT)))
+     /* tell the child of the stored tree to resize, sadly we don't know
+	the handle of that window, since the helper handles it for us */
+     if (!XQueryTree(display, *window, &root, &parent, &children, &num_children))
      {
-	  /* tell the child of the stored tree to resize, sadly we don't know
-	     the handle of that window, since the helper handles it for us */
-	  if (!XQueryTree(display, *window, &root, &parent, &children, &num_children))
-	  {
-	       D("Querytree failed!!! using current window\n");
-	       useme = window;
-	  }
-	  if (children == NULL) {
-	       D("No children!!! using current window\n");
-	       useme = window;
-	  } else if (num_children > 1) {
-	       D("Multiple Children, Not Resizing");
-	       XFree(children);
-	       return 1;
-	  } else {
-	       /* use the first child */
-	       useme = &children[0];
-	  }
-
-	  if (flags & H_FILL) {
-	       fill_window(display, *useme, w, h);
-	  } else if (flags & H_MAXASPECT) {
-	       max_aspect_window(display, *useme, w, h);
-	  }
-
+	  D("Querytree failed!!! using current window\n");
+	  useme = window;
+     }
+     if (children == NULL) {
+	  D("No children!!! using current window\n");
+	  useme = window;
+     } else if (num_children > 1) {
+	  D("Multiple Children, Not Resizing");
 	  XFree(children);
+	  return;
+     } else {
+	  /* use the first child */
+	  useme = &children[0];
      }
 
-     return 0;
+     if (flags & H_FILL) {
+	  fill_window(display, *useme, w, h);
+     } else if (flags & H_MAXASPECT) {
+	  max_aspect_window(display, *useme, w, h);
+     }
+
+     XFree(children);
 }
 
 /*****************************************************************************
@@ -1122,14 +1141,18 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
      THIS->display = ((NPSetWindowCallbackStruct *)window->ws_info)->display;
      THIS->displayname = XDisplayName(DisplayString(THIS->display));
      THIS->windata = *window;
-     
-     if (resize_window(THIS->flags, THIS->display, (Window *)window, window->width, window->height))
-	  return NPERR_NO_ERROR;
 
-     if (THIS->pid != -1)
+     if (THIS->mms)
      {
-	  D("Sending SIGWINCH to pid (%d)\n", THIS->pid);
-	  kill(THIS->pid, SIGWINCH);
+	  new_child(instance, THIS->mms);
+	  free(THIS->mms);
+	  THIS->mms = NULL;
+     }
+     
+     if ((THIS->flags & H_FILL) || (THIS->flags & H_MAXASPECT))
+     {
+	  resize_window(THIS->flags, THIS->display, (Window *)&window->window,
+			window->width, window->height);
      }
 
      usleep(4000);
